﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Xml;

namespace voice
{
    class VoiceSdk
    {
        // 请求基础路径
        private string apiUrl = null;

        // 主账号信息
        private string accountSid = null;

        // 用户token
        private string token = null;

        // 应用id
        private string appId = null;

        /// <summary>
        /// 设置服务器地址
        /// </summary>
        /// <param name="apiUrl">服务器地址</param>
        public void setApiUrl(string apiUrl)
        {
            this.apiUrl = apiUrl;
        }

        /// <summary>
        /// 设置主帐号信息
        /// </summary>
        /// <param name="accountSid">主帐号</param>
        public void setAccountSid(string accountSid)
        {
            this.accountSid = accountSid;
        }

        /// <summary>
        /// 设置主帐号信息
        /// </summary>
        /// <param name="token">主帐号令牌</param>
        public void setAuthToken(string accountToken)
        {
            this.token = accountToken;
        }

        /// <summary>
        /// 设置应用ID
        /// </summary>
        /// <param name="appId">应用ID</param>
        public void setAppId(string appId)
        {
            this.appId = appId;
        }

        /// <summary>
        /// 模板语音
        /// </summary>
        /// <param name="to">语音接收端手机号码集合，用英文逗号分开，每批发送的手机号数量不得超过100个</param>
        /// <param name="templateId">模板Id</param>
        /// <param name="datas">可选字段 内容数据，用于替换模板中{序号}</param>
        /// <exception cref="ArgumentNullException">参数不能为空</exception>
        /// <exception cref="Exception"></exception>
        /// <returns></returns>
        public Dictionary<string, object> sendVoice(string to, string templateId, string[] datas)
        {
            Dictionary<string, object> initError = check();
            if (initError != null)
            {
                return initError;
            }
            initError = checkAccount();
            if (initError != null)
            {
                return initError;
            }
            initError = checkAppId();
            if (initError != null)
            {
                return initError;
            }
            initError = checkTo(to);
            if (initError != null)
            {
                return initError;
            }
            initError = checkTemplateId(templateId);
            if (initError != null)
            {
                return initError;
            }

            try
            {
                string date = DateTime.Now.ToString("yyyyMMddhhmmss");

                //构建URL内容
                string sig = MD5Encrypt(accountSid + token + date);
                string sendUrl = string.Format("{0}voice/submit", apiUrl);
                Uri url = new Uri(sendUrl);
                Console.WriteLine(url);
                Encoding encode = Encoding.GetEncoding("utf-8");
                byte[] enByte = encode.GetBytes(accountSid + ":" + date);
                //认证字符串
                string authorization = Convert.ToBase64String(enByte);

                //创建网络请求
                HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
                //构建Head
                request.Method = "POST";
                // 将认证校验的字符串放到请求头
                request.Headers.Add("Authorization", authorization);
                request.Headers.Add("sig",sig);

                //构建Body
                StringBuilder bodyData = new StringBuilder();
                request.Accept = "application/json";
                request.ContentType = "application/json;charset=utf-8";
                bodyData.Append("{");
                bodyData.Append("\"to\":\"").Append(to).Append("\"");
                bodyData.Append(",\"appId\":\"").Append(appId).Append("\"");
                bodyData.Append(",\"templateId\":\"").Append(templateId).Append("\"");
                if (datas != null && datas.Length > 0)
                {
                    bodyData.Append(",\"datas\":[");
                    int index = 0;
                    foreach (string item in datas)
                    {
                        if (index == 0)
                        {
                            bodyData.Append("\"" + item + "\"");
                        }
                        else
                        {
                            bodyData.Append(",\"" + item + "\"");
                        }
                        index++;
                    }
                    bodyData.Append("]");
                }
                bodyData.Append("}");

                byte[] byteData = UTF8Encoding.UTF8.GetBytes(bodyData.ToString());
                Console.WriteLine("SendVoiceTemplate requestBody = " + bodyData.ToString());
                //开始请求
                using (Stream postStream = request.GetRequestStream())
                {
                    postStream.Write(byteData, 0, byteData.Length);
                }
                //获取请求
                using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
                {
                    // Get the response stream  
                    StreamReader reader = new StreamReader(response.GetResponseStream());
                    // 读取返回字符串
                    string responseStr = reader.ReadToEnd();

                    Console.WriteLine("SendVoiceTemplate responseBody = " + responseStr);
                    if (responseStr != null && responseStr.Length > 0)
                    {
                        Dictionary<string, object> responseResult = new Dictionary<string, object> ();
                        responseResult["resposeBody"] = responseStr;
                        return responseResult;
                    }
                    return new Dictionary<string, object> {
                        { "code", "DH:0010" },
                        { "msg", "【服务器】服务器请求出错，请确认参数是否正确，如无法解决，请联系工作人员" },
                        { "time", null },
                        { "msgid",""}
                    };
                }
            }
            catch (Exception e)
            {
                throw e;
            }
        }


        /// <summary>
        /// 语音状态报告
        /// </summary>
        public Dictionary<string, object> getReport()
        {
            // 检查账号信息
            Dictionary<string, object> initError = checkAccount();
            if (initError != null)
            {
                return initError;
            }
            try
            {
                string date = DateTime.Now.ToString("yyyyMMddhhmmss");
                //构建URL内容
                string sig = MD5Encrypt(accountSid + token + date);
                string sendUrl = string.Format("{0}voice/report", apiUrl);
                Uri url = new Uri(sendUrl);
                Console.WriteLine(url);
                Encoding encode = Encoding.GetEncoding("utf-8");
                byte[] enByte = encode.GetBytes(accountSid + ":" + date);
                //认证字符串
                string authorization = Convert.ToBase64String(enByte);
                //创建网络请求
                HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
                //构建Head
                request.Method = "POST";
                // 将认证校验的字符串放到请求头
                request.Headers.Add("authorization", authorization);
                request.Headers.Add("sig", sig);
                //构建Body
                request.Accept = "application/json";
                request.ContentType = "application/json;charset=utf-8";
                //获取请求
                using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
                {
                    // Get the response stream  
                    StreamReader reader = new StreamReader(response.GetResponseStream());
                    // 读取返回字符串
                    string responseStr = reader.ReadToEnd();

                    Console.WriteLine("rquestVoiceReport responseBody = " + responseStr);
                    if (responseStr != null && responseStr.Length > 0)
                    {
                        Dictionary<string, object> responseResult = new Dictionary<string, object>();
                        responseResult["resposeBody"] = responseStr;
                        return responseResult;
                    }
                    return new Dictionary<string, object> {
                        { "code", "DH:0010" },
                        { "msg", "【服务器】服务器请求出错，请确认参数是否正确，如无法解决，请联系工作人员" },
                        { "datas", null }
                    };
                }
            }
            catch (Exception e)
            {
                throw e;
            }
        }



        /// <summary>
        /// 检查服务器地址信息
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, object> check()
        {
            string code = "DH:0000";
            string msg = null;

            if (apiUrl == null)
            {
                code = "DH:0019";
                msg = "【语音】url空";
            }

            if (code != "DH:0000")
            {
                return new Dictionary<string, object>
                {
                    { "code", code},
                    { "msg", msg }
                };
            }
            return null;
        }
        /// <summary>
        /// 检查主帐号信息
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, object> checkAccount()
        {
            string code = "DH:0000";
            string msg = null;

            if (accountSid == null)
            {
                code = "DH:0017";
                msg = "【账号】帐号空";
            }
            else if (token == null)
            {
                code = "DH:0018";
                msg = "【账号】帐号令牌空 ";
            }

            if (code != "DH:0000")
            {
                return new Dictionary<string, object>
                {
                    { "code", code},
                    { "msg", msg }
                };
            }
            return null;
        }

        /// <summary>
        /// 检查应用ID
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, object> checkAppId()
        {
            if (appId == null)
            {
                return new Dictionary<string, object> {
                    { "code", "DH:0016" },
                    { "msg", "【应用】应用ID为空" },
                    { "time",null },
                    { "msgid",null }
                };
            }
            return null;
        }

        private Dictionary<string, object> checkTo(string to)
        {
            if (to == null)
            {
                return new Dictionary<string, object> {
                    { "code", "000006" },
                    { "msg", "【语音】接收电话的号码为空" },
                    { "time",null },
                    { "msgid",null }
                };
            }
            return null;
        }

        private Dictionary<string, object> checkTemplateId(string templateId)
        {
            if (templateId == null)
            {
                return new Dictionary<string, object> {
                    { "code", "000008" },
                    { "msg", "【语言】语音模板为空或未设置" },
                    { "time",null },
                    { "msgid",null }
                };
            }
            return null;
        }

        /// <summary>
        /// MD5加密
        /// </summary>
        /// <param name="source">原内容</param>
        /// <returns>加密后内容</returns>
        public static string MD5Encrypt(string source)
        {
            // Create a new instance of the MD5CryptoServiceProvider object.
            System.Security.Cryptography.MD5 md5Hasher = System.Security.Cryptography.MD5.Create();

            // Convert the input string to a byte array and compute the hash.
            byte[] encodeByte = md5Hasher.ComputeHash(Encoding.Default.GetBytes(source));

            // Create a new Stringbuilder to collect the bytes and create a string.
            StringBuilder sBuilder = new StringBuilder();

            // Loop through each byte of the hashed data and format each one as a hexadecimal string.
            for (int i = 0; i < encodeByte.Length; i++)
            {
                sBuilder.Append(encodeByte[i].ToString("X2"));
            }
            // Return the hexadecimal string.
            return sBuilder.ToString();
        }
    }
}
